class TextAnalysisHelpersException(Exception):
    """Base text analysis helpers exception"""

    pass


class WebPageDownloadError(TextAnalysisHelpersException):
    """Exception that is raised if there was an error while downloading the
    contents of a web page"""

    def __init__(
        self, message=None, url=None, status_code=None, response=None
    ):
        """Create a new WebPageDownloadError object

        :param str|None message: the description of the error
        :param str|None url: the url that caused the error to occur
        :param int|None status_code: the response status code
        :param str|None response: the response text
        """
        super(WebPageDownloadError, self).__init__(
            message, url, status_code, response
        )

        self.message = message
        self.url = url
        self.status_code = status_code
        self.response = response



from collections import defaultdict
from itertools import combinations_with_replacement


class Rake:
    """RAKE keyword extractor"""

    def __init__(
        self, word_tokenizer, sentence_tokenizer, stop_words, delimiters
    ):
        """Create a new Rake objects

        :param func word_tokenizer: a callable that splits a sentence into a
            list of words
        :param func sentence_tokenizer: a callable tat splits the text into
            sentences
        :param list[str] stop_words: a list of stop words to use
        :param list[str] delimiters: the list of word delimiters
        """
        self._word_tokenizer = word_tokenizer
        self._sentence_tokenizer = sentence_tokenizer
        self._delimiters = delimiters

        self._stop_words = set()
        for stop_word in stop_words:
            self._stop_words.add(stop_word)
            splited_stop_word = word_tokenizer(stop_word)
            self._stop_words.update(splited_stop_word)

    def _extract_candidate_keywords(self, tokenized_document):
        """Extract the candidate keywords from the text

        :param list[list[str]] tokenized_document: the tokenized document
        :rtype: list[list[str]]
        :return: the candidate keywords
        """
        candidate_keywords = []

        for sentence in tokenized_document:
            keyword = []
            for word in sentence:
                if not word:
                    continue

                if self._is_delimiter(word):
                    if keyword:
                        candidate_keywords.append(keyword)
                        keyword = []
                else:
                    keyword.append(word)

            if keyword:
                candidate_keywords.append(keyword)

        return candidate_keywords

    def _is_stop_word(self, word):
        """Check if this word is a stop word

        :param str word: the word to check
        :rtype: bool
        :return True if this is a stop word
        """
        return word.lower() in self._stop_words

    def _is_delimiter(self, word):
        """Check if this word is a delimiter

        :param str word: the word to check
        :rtype: bool
        :return True if this word is a delimiter
        """
        return self._is_stop_word(word) or word in self._delimiters

    def _tokenize_document(self, document):
        """Tokenize the given document in a list of tokenized sentences

        :param str document:
        :rtype: list[list[str]]
        :return: return the tokenized document
        """
        tokenized_document = []
        sentences = self._sentence_tokenizer(document)
        for sentence in sentences:
            words = self._word_tokenizer(sentence)
            tokenized_document.append(words)

        return tokenized_document

    def _calculate_word_co_occurrences(self, candidate_keywords):
        """Calculate the word cooccurrences for the candidate keywords

        :param list[list[str]] candidate_keywords: the keywords candidates
        :rtype: dict[str, dict[str, int]]
        :return: return the word cooccurrence matrix
        """
        word_co_occurrences = defaultdict(lambda: defaultdict(int))
        for candidate in candidate_keywords:
            for w1, w2 in combinations_with_replacement(candidate, 2):
                w1 = w1.lower()
                w2 = w2.lower()
                word_co_occurrences[w1][w2] += 1
                if w1 != w2:
                    word_co_occurrences[w2][w1] += 1

        return word_co_occurrences

    def _calculate_word_scores(self, word_co_occurrences):
        """Calculate the word score

        :param dict[str, dict[str, int]] word_co_occurrences: the word
            cooccurrences matrix
        :rtype: dict[str, float]
        :return: return a dictionary with the word scores
        """
        word_scores = {}
        for word in word_co_occurrences:
            deg = sum(word_co_occurrences[word].values())
            freq = word_co_occurrences[word][word]
            word_scores[word] = deg / freq

        return word_scores

    def _calculate_candidate_keyword_scores(
        self, candidate_keywords, word_scores
    ):
        """Calculate the candidate keyword scores

        :param list[list[str]] candidate_keywords: the list of candidate
            keywords
        :param dict[str, float] word_scores: the word scores
        :rtype: dict[tuple, float]
        :return: return the candidate keyword scores
        """
        candidate_scores = defaultdict(float)
        for candidate in candidate_keywords:
            for word in candidate:
                candidate_scores[tuple(candidate)] += word_scores[word.lower()]

        return candidate_scores

    def _find_candidate_keyword_aliases(self, candidate_keywords):
        """Find the keyword aliases

        A keyword alias is basically just another form of the keyword that
        contains characters that might differ because they are lowercase or
        uppercase.

        :param list[list[str]] candidate_keywords: the keywords
        :rtype: dict[tuple, list[tuple]]
        :return: a dictionary with the aliases mapping
        """
        candidate_keyword_aliases = defaultdict(set)
        for candidate in candidate_keywords:
            normalised_candidate = tuple([word.lower() for word in candidate])
            candidate_keyword_aliases[normalised_candidate].add(
                tuple(candidate)
            )

        return candidate_keyword_aliases

    def _calculate_candidate_keyword_aliases_scores(
        self, candidate_keyword_aliases, candidate_scores
    ):
        """Calculate the scores of the keyword aliases

        :param dict[tuple, list[tuple]] candidate_keyword_aliases: the keyword
            aliases
        :param dict[tuple float] candidate_scores: the keyword scores
        :rtype: dict[tuple, float]
        :return: return the calculated keyword alias scores
        """
        normalized_candidates_scores = defaultdict(float)
        for candidate_keyword_alias in candidate_keyword_aliases.values():
            primary_candidate, *candidate_aliases = candidate_keyword_alias
            normalized_candidates_scores[primary_candidate] = candidate_scores[
                primary_candidate
            ]

            for candidate_alias in candidate_aliases:
                normalized_candidates_scores[
                    primary_candidate
                ] += candidate_scores[candidate_alias]

        return normalized_candidates_scores

    def extract_keywords(self, document):
        """Extract the keywords from the given document text

        :param document: the document text
        :rtype: list[str]
        :return: a dictionary with the keywords. The key is the keyword and the
            value is the keyword score
        """
        tokenized_document = self._tokenize_document(document)
        candidate_keywords = self._extract_candidate_keywords(
            tokenized_document
        )
        word_co_occurrences = self._calculate_word_co_occurrences(
            candidate_keywords
        )
        word_scores = self._calculate_word_scores(word_co_occurrences)
        candidate_scores = self._calculate_candidate_keyword_scores(
            candidate_keywords, word_scores
        )
        candidate_keyword_aliases = self._find_candidate_keyword_aliases(
            candidate_keywords
        )
        candidate_scores = self._calculate_candidate_keyword_aliases_scores(
            candidate_keyword_aliases, candidate_scores
        )

        return {
            " ".join(candicate_keyword): score
            for candicate_keyword, score in candidate_scores.items()
        }



import json
from abc import ABCMeta, abstractmethod
from collections import namedtuple

from text_analysis_helpers.helpers import current_date

TextStatistics = namedtuple(
    "TextStatistics",
    [
        "sentence_count",
        "word_count",
        "mean_sentence_word_count",
        "median_sentence_word_count",
        "min_sentence_word_count",
        "max_sentence_word_count",
        "average_sentence_word_count",
        "sentence_word_count_std",
        "sentence_word_count_variance",
    ],
)

WebPage = namedtuple("WebPage", ["url", "html"])

SocialNetworkData = namedtuple("SocialNetworkData", ["opengraph", "twitter"])


class BaseAnalysisResult(metaclass=ABCMeta):
    """Base model for all analysis results"""

    def __init__(self):
        creation_date = current_date()

        self.created_at = creation_date.datetime
        self.created_at_timestamp = creation_date.timestamp

    def save(self, output_file):
        """Encode to json and save to a file

        :param str output_file: the output file
        """
        with open(output_file, "w") as f:
            f.write(self.as_json())

    @abstractmethod
    def as_dict(self):
        """Convert the analysis result object into a dictionary

        :rtype: dict
        :return: the result object data as a dictionary
        """
        return {
            "created_at": self.created_at.strftime("%Y-%m-%d %H:%M:%S %z"),
            "created_at_timestamp": self.created_at_timestamp,
        }

    def as_json(self):
        """Convert the analysis result object into a json string

        :rtype: str
        :return: the data encoded in json
        """
        return json.dumps(self.as_dict())


class TextAnalysisResult(BaseAnalysisResult):
    """Text analysis result"""

    def __init__(
        self,
        text,
        keywords,
        readability_scores,
        statistics,
        summary,
        named_entities,
    ):
        """Create a new TextAnalysisResult object

        :param str text: the text that was analysed
        :param dict[str, int] keywords: the extracted keywords and their scores
        :param dict[str, T] readability_scores: the readability scores
        :param TextStatistics statistics: the text statistics
        :param str summary: the text summary
        :param dict[str, set[str]] named_entities: the extracted named entities
        """
        super(TextAnalysisResult, self).__init__()

        self.text = text
        self.keywords = keywords
        self.readability_scores = readability_scores
        self.statistics = statistics
        self.summary = summary
        self.named_entities = named_entities

    def as_dict(self):
        data = super(TextAnalysisResult, self).as_dict()

        named_entities = {
            named_entity_type: list(self.named_entities[named_entity_type])
            for named_entity_type in self.named_entities
        }

        data.update(
            {
                "text": self.text,
                "keywords": self.keywords,
                "readability_scores": self.readability_scores,
                "statistics": dict(self.statistics._asdict()),
                "summary": self.summary,
                "named_entities": named_entities,
            }
        )

        return data


class HtmlAnalysisResult(TextAnalysisResult):
    """Html analysis result"""

    def __init__(self, url, html, title, social_network_data, text_data):
        """Create a new HtmlAnalysisResult object

        :param str url: the web page url
        :param str html: the web page content
        :param str title: the web page title
        :param SocialNetworkData social_network_data: the extracted social
            network data
        :param TextAnalysisResult text_data: the text analysis result for the
            text that was extracted from the web page
        """
        super(HtmlAnalysisResult, self).__init__(
            text=text_data.text,
            keywords=text_data.keywords,
            readability_scores=text_data.readability_scores,
            statistics=text_data.statistics,
            summary=text_data.summary,
            named_entities=text_data.named_entities,
        )

        self.url = url
        self.html = html
        self.title = title
        self.social_network_data = social_network_data

        self._extract_images(social_network_data)
        self._extract_videos(social_network_data)

    def _extract_images(self, social_network_data):
        self.images = []
        twitter_data = social_network_data.twitter or {}

        image = twitter_data.get("image")
        if image:
            self.images.append(image)
        image = twitter_data.get("image:src")
        if image and image not in self.images:
            self.images.append(image)

        for opengraph_item in social_network_data.opengraph or []:
            for property_name, value in opengraph_item.get("properties", []):
                if property_name == "og:image" and value not in self.images:
                    self.images.append(value)

    def _extract_videos(self, social_network_data):
        self.movies = []

        for opengraph_item in social_network_data.opengraph or []:
            for property_name, value in opengraph_item.get("properties", []):
                if property_name == "og:video" and value not in self.movies:
                    self.movies.append(value)

    def as_dict(self):
        data = super(HtmlAnalysisResult, self).as_dict()

        data.update(
            {
                "url": self.url,
                "html": self.html,
                "title": self.title,
                "images": list(self.images),
                "movies": self.movies,
                "social_network_data": {
                    "twitter": self.social_network_data.twitter,
                    "opengraph": self.social_network_data.opengraph,
                },
            }
        )

        return data



from nltk import sent_tokenize, word_tokenize
from nltk.corpus import stopwords
from sumy.nlp.stemmers import Stemmer
from sumy.nlp.tokenizers import Tokenizer
from sumy.parsers.plaintext import PlaintextParser
from sumy.summarizers.lsa import LsaSummarizer as Summarizer
from sumy.utils import get_stop_words
from textstat.textstat import textstat

from text_analysis_helpers.keywords.rake import Rake


def extract_keywords(text):
    rake = Rake(
        word_tokenizer=word_tokenize,
        sentence_tokenizer=sent_tokenize,
        stop_words=stopwords.words("english"),
        delimiters=[",", "’", "‘", "“", "”", "“", "?", "—", "."],
    )

    return rake.extract_keywords(text)


def calculate_readability_scores(text):
    score_functions = [
        "flesch_reading_ease",
        "smog_index",
        "flesch_kincaid_grade",
        "coleman_liau_index",
        "automated_readability_index",
        "dale_chall_readability_score",
        "difficult_words",
        "linsear_write_formula",
        "gunning_fog",
        "text_standard",
    ]

    return {
        score_function: getattr(textstat, score_function)(text)
        for score_function in score_functions
    }


def create_summary(text, language="english", sentence_count=5):
    parser = PlaintextParser.from_string(text, Tokenizer(language))
    stemmer = Stemmer(language)
    summarizer = Summarizer(stemmer)
    summarizer.stop_words = get_stop_words(language)

    return " ".join(
        [
            str(sentence)
            for sentence in summarizer(parser.document, sentence_count)
        ]
    )



import itertools
from collections import defaultdict

import numpy as np
from nltk import sent_tokenize, word_tokenize
from nltk.data import load as nltk_data_load
from nltk.tag.perceptron import PerceptronTagger
from nltk.tree import Tree

from text_analysis_helpers.models import TextAnalysisResult, TextStatistics
from text_analysis_helpers.processors.text import (
    calculate_readability_scores,
    create_summary,
    extract_keywords,
)


class TextAnalyser(object):
    """Text analyser"""

    MULTICLASS_NE_CHUNKER = (
        "chunkers/maxent_ne_chunker/english_ace_multiclass.pickle"
    )

    def __init__(self):
        self.__pos_tagger = PerceptronTagger()
        self.__ne_chunker = nltk_data_load(self.MULTICLASS_NE_CHUNKER)

    def analyse_file(self, filename):
        """Analyse the contents of a file

        :param str filename: the path to a file
        :rtype: TextAnalysisResult
        :return: the analysis result
        """
        with open(filename, "r") as f:
            return self.analyse(f.read())

    def _calculate_text_statistics(self, sentences, sentence_words):
        """Calculate the text statistics

        :param list[str] sentences: a list with the text sentences
        :param list[list[str]] sentence_words: a list with the sentences that
            have been tokenized into separate words
        :rtype: TextStatistics
        :return: the calculated text statistics
        """
        words = list(itertools.chain(*sentence_words))

        sentence_word_counts = np.array(
            [len(sentence) for sentence in sentence_words]
        )

        return TextStatistics(
            sentence_count=len(sentences),
            word_count=len(words),
            mean_sentence_word_count=float(sentence_word_counts.mean()),
            median_sentence_word_count=float(np.median(sentence_word_counts)),
            min_sentence_word_count=int(sentence_word_counts.min()),
            max_sentence_word_count=int(sentence_word_counts.max()),
            average_sentence_word_count=float(
                np.average(sentence_word_counts)
            ),
            sentence_word_count_std=float(sentence_word_counts.std()),
            sentence_word_count_variance=float(sentence_word_counts.var()),
        )

    def _extract_named_entities(self, sentence_words):
        """Extract the named entities from the sentences

        The result from this method is a dictionary with the named entity types
        as keys and a set of the names entities as values

        :param list[list[str]] sentence_words: a list with the sentences that
            have been tokenized into separate words
        :rtype: dict[str: set]
        :return: the extracted dictionary words
        """
        tagged_sentences = [
            self.__pos_tagger.tag(sentence) for sentence in sentence_words
        ]
        chunked_sentences = [
            self.__ne_chunker.parse(sentence) for sentence in tagged_sentences
        ]

        named_entities = defaultdict(set)
        for sentence in chunked_sentences:
            for item in sentence:
                if isinstance(item, Tree):
                    ne_type = item.label()
                    entity = " ".join(
                        [
                            entity_component[0]
                            for entity_component in item.leaves()
                        ]
                    )

                    named_entities[ne_type].add(entity)

        return dict(named_entities)

    def analyse(self, text):
        """Analyse the given text

        :param str text: the text to analyse
        :rtype: TextAnalysisResult
        :return: the analysis result
        """
        readability_scores = calculate_readability_scores(text)

        keywords = None
        if isinstance(text, str) and len(text) != 0:
            keywords = extract_keywords(
                text=text,
            )

        sentences = sent_tokenize(text)
        sentence_words = [word_tokenize(sentence) for sentence in sentences]

        statistics = self._calculate_text_statistics(sentences, sentence_words)
        summary = create_summary(text)
        named_entities = self._extract_named_entities(sentence_words)

        return TextAnalysisResult(
            text=text,
            keywords=keywords,
            readability_scores=readability_scores,
            statistics=statistics,
            summary=summary,
            named_entities=named_entities,
        )



import logging

import extruct
from bs4 import BeautifulSoup

from text_analysis_helpers.downloaders import download_web_page
from text_analysis_helpers.models import HtmlAnalysisResult, SocialNetworkData
from text_analysis_helpers.processors.html import (
    extract_page_content,
    extract_page_data,
    extract_twitter_card,
)
from text_analysis_helpers.text import TextAnalyser

logger = logging.getLogger(__name__)


class HtmlAnalyser(object):
    """Html content analyser"""

    def __init__(self):
        self.__text_analyser = TextAnalyser()

    def analyse_url(self, url, timeout=5, headers=None, verify=True):
        """Download and analyse the contents of the given url

        :param str url: the url to analyse
        :param int timeout: the request timeout
        :param dict|None headers: the headers to add to the request
        :param boolean verify: verify ssl
        :rtype: HtmlAnalysisResult
        :return: the analysis result
        """
        web_page = download_web_page(
            url=url, timeout=timeout, headers=headers, verify=verify
        )

        return self.analyse(web_page)

    def analyse(self, web_page):
        """Analyse the web page contents

        :param text_analysis_helpers.models.WebPage web_page: the wb page
            contents
        :rtype: HtmlAnalysisResult
        :return: the analysis result
        """
        page_content = extract_page_content(web_page.html)
        text_analysis_result = self.__text_analyser.analyse(page_content)
        soup = BeautifulSoup(web_page.html, "html.parser")
        page_data = extract_page_data(soup)
        extracted_data = extruct.extract(web_page.html, base_url=web_page.url)
        twitter_card = extract_twitter_card(soup)

        return HtmlAnalysisResult(
            url=web_page.url,
            html=web_page.html,
            title=page_data["title"],
            social_network_data=SocialNetworkData(
                opengraph=extracted_data.get("opengraph"), twitter=twitter_card
            ),
            text_data=text_analysis_result,
        )



from argparse import ArgumentParser

from text_analysis_helpers.html import HtmlAnalyser
from text_analysis_helpers.text import TextAnalyser


def analyse_url(args):
    analyser = HtmlAnalyser()
    analysis_result = analyser.analyse_url(args.url)
    analysis_result.save(args.output)


def analyse_file(args):
    analyser = TextAnalyser()
    analysis_result = analyser.analyse_file(args.filename)
    analysis_result.save(args.output)


def get_arguments():
    parser = ArgumentParser()
    subparsers = parser.add_subparsers()

    url_parser = subparsers.add_parser(
        "analyse-url",
        description="analyse the contents of a url",
        help="analyse the contents of a url",
    )

    url_parser.add_argument(
        "--output",
        default="analysis_result.json",
        help="the name of the file in which to save the result",
    )

    url_parser.add_argument("url", help="the url to analyse")
    url_parser.set_defaults(func=analyse_url)

    file_parser = subparsers.add_parser(
        "analyse-file",
        description="analyse the contents of a file",
        help="analyse the contents of a file",
    )

    file_parser.add_argument(
        "--output",
        default="analysis_result.json",
        help="the name of the file in which to save the result",
    )

    file_parser.add_argument("filename", help="the file to analyse")
    file_parser.set_defaults(func=analyse_file)

    return parser.parse_args()


def main():
    args = get_arguments()
    args.func(args)



from logging import getLogger

from articles.mss.extractors import MSSArticleExtractor

logger = getLogger(__name__)


def extract_page_content(html_content):
    article_extractor = MSSArticleExtractor()

    return article_extractor.extract_article(html_content)


def extract_page_data(soup):
    title = soup.find("title")

    return {"title": title.text if title else None}


def extract_twitter_card(soup):
    card = {}

    for meta in soup.find_all("meta"):
        name = meta.get("name", "")
        if name.startswith("twitter:"):
            items = name.split(":")
            if len(items) < 2:
                msg = "Invalid twitter card value: twitter_card(%s)"
                logger.warning(msg, name)
                continue
            card[":".join(items[1:])] = meta.get("content")

    # if twitter card data could not be extracted then return None instead
    # of an empty dictionary
    if len(card) == 0:
        logger.warning("failed to extract twitter card")
        card = None

    return card



import requests

from text_analysis_helpers.exceptions import WebPageDownloadError
from text_analysis_helpers.models import WebPage


def download_web_page(url, timeout=5, **kwargs):
    """Download a web page

    :param str url: the url of the web page
    :param int timeout: the request timeout
    :param kwargs: additional arguments to pass to the `requests.get`
        method
    :rtype: WebPage
    :return: the web page contents
    """
    _kwargs = {"timeout": timeout}
    _kwargs.update(kwargs)

    response = requests.get(url, **_kwargs)

    if response.status_code < 200 or response.status_code >= 300:
        raise WebPageDownloadError(
            message="failed to download web page",
            url=url,
            status_code=response.status_code,
            response=response.text,
        )

    return WebPage(url=url, html=response.text)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['text_analysis_helpers',
 'text_analysis_helpers.keywords',
 'text_analysis_helpers.processors']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=0.15.5,<1.0.0',
 'article-extraction>=0.3.0,<0.4.0',
 'extruct>=0.13.0,<1.0.0',
 'nltk>=3.3,<4.0',
 'numpy>=1.15.2,<2.0.0',
 'requests>=2.26.0,<3.0.0',
 'sumy>=0.11.0,<1.0.0',
 'textstat>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['text-analysis-helpers-cli = '
                     'text_analysis_helpers.cli:main']}

setup_kwargs = {
    'name': 'text-analysis-helpers',
    'version': '0.5.0',
    'description': 'Collection of classes and functions for text analysis',
    'long_description': '# Introduction\n\nText-analysis-helpers is a collection of classes and functions for text analysis.\n\n# Installation\n\nA Python 3 interpreter is required. It is recommended to install the package in\na virtual environment in order to avoid corrupting the system\'s Python interpreter\npackages.\n\nInstall the package using pip.\n\n```bash\npip install text-analysis-helpers\n\npython -m nltk.downloader "punkt"\npython -m nltk.downloader "averaged_perceptron_tagger"\npython -m nltk.downloader "maxent_ne_chunker"\npython -m nltk.downloader "words"\npython -m nltk.downloader "stopwords"\n```\n\n# Usage\n\nYou can use the HtmlAnalyser object to analyse the contents of a url.\n\n```python\nfrom text_analysis_helpers.html import HtmlAnalyser\n\nanalyser = HtmlAnalyser()\nanalysis_result = analyser.analyse_url("https://www.bbc.com/sport/formula1/64983451")\n\nanalysis_result.save("analysis_result.json")\n```\n\nYou can see the scripts in the `examples` folder for some usage examples.\n\nThere is also an cli utility that can be used to analyse a url. For example to\nanalyse a url and save the analysis result to a json encoded file execute the\nfollowing command in the terminal.\n\n```bash\ntext-analysis-helpers-cli analyse-url --output analysis_result.json https://www.bbc.com/sport/formula1/64983451\n```\n',
    'author': 'Matigakis Panagiotis',
    'author_email': 'pmatigakis@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pmatigakis/text-analysis-helpers',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
